@@ -2,38 +2,36 @@ package createcluster
 
 import (
 	"bytes"
-	"encoding/json"
 	"fmt"
 	"io/ioutil"
-	"net/http"
 	"os"
 	"os/user"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
-
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/cli-runtime/pkg/printers"
 	"k8s.io/client-go/kubernetes/scheme"
-	"k8s.io/client-go/tools/clientcmd"
-	"sigs.k8s.io/controller-runtime/pkg/client/config"
 
+	"github.com/openshift/hive/contrib/pkg/utils"
 	awsutils "github.com/openshift/hive/contrib/pkg/utils/aws"
 	azurecredutil "github.com/openshift/hive/contrib/pkg/utils/azure"
 	gcputils "github.com/openshift/hive/contrib/pkg/utils/gcp"
 	openstackutils "github.com/openshift/hive/contrib/pkg/utils/openstack"
+	ovirtutils "github.com/openshift/hive/contrib/pkg/utils/ovirt"
 	"github.com/openshift/hive/pkg/apis"
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
 	"github.com/openshift/hive/pkg/clusterresource"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/gcpclient"
-	"github.com/openshift/hive/pkg/resource"
+	"github.com/openshift/installer/pkg/validate"
 )
 
 const longDesc = `
@@ -72,12 +70,12 @@ AWS credentials. These are only relevant for creating a cluster on AWS. If
 --creds-file is used it will take precedence over these environment
 variables.
 
-GOVC_USERNAME and GOVC_PASSWORD - Are used to determine your vSphere 
+GOVC_USERNAME and GOVC_PASSWORD - Are used to determine your vSphere
 credentials.
 GOVC_TLS_CA_CERTS - Is used to provide CA certificates for communicating
 with the vSphere API.
 GOVC_NETWORK, GOVC_DATACENTER, GOVC_DATASTORE and GOVC_HOST (vCenter host)
-can be used as alternatives to the associated commandline argument. 
+can be used as alternatives to the associated commandline argument.
 These are only relevant for creating a cluster on vSphere.
 
 RELEASE_IMAGE - Release image to use to install the cluster. If not specified,
@@ -92,6 +90,7 @@ const (
 	cloudGCP             = "gcp"
 	cloudOpenStack       = "openstack"
 	cloudVSphere         = "vsphere"
+	cloudOVirt           = "ovirt"
 
 	testFailureManifest = `apiVersion: v1
 kind: NotARealSecret
@@ -109,6 +108,7 @@ var (
 		cloudGCP:       true,
 		cloudOpenStack: true,
 		cloudVSphere:   true,
+		cloudOVirt:     true,
 	}
 )
 
@@ -128,6 +128,8 @@ type Options struct {
 	ReleaseImage             string
 	ReleaseImageSource       string
 	DeleteAfter              string
+	HibernateAfter           string
+	HibernateAfterDur        *time.Duration
 	ServingCert              string
 	ServingCertKey           string
 	UseClusterImageSet       bool
@@ -147,6 +149,13 @@ type Options struct {
 	AdoptAdminUsername       string
 	AdoptAdminPassword       string
 	MachineNetwork           string
+	Region                   string
+	Labels                   []string
+	SkipMachinePools         bool
+	AdditionalTrustBundle    string
+
+	// AWS
+	AWSUserTags []string
 
 	// Azure
 	AzureBaseDomainResourceGroupName string
@@ -169,12 +178,21 @@ type Options struct {
 	VSphereNetwork          string
 	VSphereCACerts          string
 
+	// Ovirt
+	OvirtClusterID       string
+	OvirtStorageDomainID string
+	OvirtNetworkName     string
+	OvirtAPIVIP          string
+	OvirtIngressVIP      string
+	OvirtCACerts         string
+
 	homeDir string
+	log     log.FieldLogger
 }
 
 // NewCreateClusterCommand creates a command that generates and applies cluster deployment artifacts.
 func NewCreateClusterCommand() *cobra.Command {
-	opt := &Options{}
+	opt := &Options{log: log.WithField("command", "create-cluster")}
 
 	opt.homeDir = "."
 
@@ -187,7 +205,7 @@ func NewCreateClusterCommand() *cobra.Command {
 	if _, err := os.Stat(defaultPullSecretFile); os.IsNotExist(err) {
 		defaultPullSecretFile = ""
 	} else if err != nil {
-		log.WithError(err).Errorf("%v can not be used", defaultPullSecretFile)
+		opt.log.WithError(err).Errorf("%v can not be used", defaultPullSecretFile)
 	}
 
 	cmd := &cobra.Command{
@@ -196,22 +214,22 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=aws
 create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=azure --azure-base-domain-resource-group-name=RESOURCE_GROUP_NAME
 create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=gcp
 create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=openstack --openstack-api-floating-ip=192.168.1.2 --openstack-cloud=mycloud
-create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=vsphere --vsphere-vcenter=vmware.devcluster.com --vsphere-datacenter=dc1 --vsphere-default-datastore=nvme-ds1 --vsphere-api-vip=192.168.1.2 --vsphere-ingress-vip=192.168.1.3 --vsphere-cluster=devel --vsphere-network="VM Network" --vsphere-ca-certs=/path/to/cert`,
+create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=vsphere --vsphere-vcenter=vmware.devcluster.com --vsphere-datacenter=dc1 --vsphere-default-datastore=nvme-ds1 --vsphere-api-vip=192.168.1.2 --vsphere-ingress-vip=192.168.1.3 --vsphere-cluster=devel --vsphere-network="VM Network" --vsphere-ca-certs=/path/to/cert
+create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=ovirt --ovirt-api-vip 192.168.1.2 --ovirt-dns-vip 192.168.1.3 --ovirt-ingress-vip 192.168.1.4 --ovirt-network-name ovirtmgmt --ovirt-storage-domain-id 00000000-e77a-456b-uuid --ovirt-cluster-id 00000000-8675-11ea-uuid --ovirt-ca-certs ~/.ovirt/ca`,
 		Short: "Creates a new Hive cluster deployment",
 		Long:  fmt.Sprintf(longDesc, defaultSSHPublicKeyFile, defaultPullSecretFile),
 		Args:  cobra.ExactArgs(1),
 		Run: func(cmd *cobra.Command, args []string) {
 			log.SetLevel(log.InfoLevel)
 			if err := opt.Complete(cmd, args); err != nil {
-				return
+				opt.log.WithError(err).Fatal("Error")
 			}
 			if err := opt.Validate(cmd); err != nil {
-				return
+				opt.log.WithError(err).Fatal("Error")
 			}
 			err := opt.Run()
 			if err != nil {
-				log.WithError(err).Error("Error")
-				os.Exit(1)
+				opt.log.WithError(err).Fatal("Error")
 			}
 		},
 	}
@@ -225,6 +243,7 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=vsphere --vsphere-vcenter=vmware.
 	flags.StringVar(&opt.BaseDomain, "base-domain", "new-installer.openshift.com", "Base domain for the cluster")
 	flags.StringVar(&opt.PullSecret, "pull-secret", "", "Pull secret for cluster. Takes precedence over pull-secret-file.")
 	flags.StringVar(&opt.DeleteAfter, "delete-after", "", "Delete this cluster after the given duration. (i.e. 8h)")
+	flags.StringVar(&opt.HibernateAfter, "hibernate-after", "", "Automatically hibernate the cluster whenever it has been running for the given duration")
 	flags.StringVar(&opt.PullSecretFile, "pull-secret-file", defaultPullSecretFile, "Pull secret file for cluster")
 	flags.StringVar(&opt.CredsFile, "creds-file", "", "Cloud credentials file (defaults vary depending on cloud)")
 	flags.StringVar(&opt.ClusterImageSet, "image-set", "", "Cluster image set to use for this cluster deployment")
@@ -243,6 +262,9 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=vsphere --vsphere-vcenter=vmware.
 	flags.BoolVar(&opt.CreateSampleSyncsets, "create-sample-syncsets", false, "Create a set of sample syncsets for testing")
 	flags.StringVar(&opt.ManifestsDir, "manifests", "", "Directory containing manifests to add during installation")
 	flags.StringVar(&opt.MachineNetwork, "machine-network", "10.0.0.0/16", "Cluster's MachineNetwork to pass to the installer")
+	flags.StringVar(&opt.Region, "region", "", "Region to which to install the cluster. This is only relevant to AWS, Azure, and GCP.")
+	flags.StringSliceVarP(&opt.Labels, "labels", "l", nil, "Label to apply to the ClusterDeployment (key=val)")
+	flags.BoolVar(&opt.SkipMachinePools, "skip-machine-pools", false, "Skip generation of Hive MachinePools for day 2 MachineSet management")
 
 	// Flags related to adoption.
 	flags.BoolVar(&opt.Adopt, "adopt", false, "Enable adoption mode for importing a pre-existing cluster into Hive. Will require additional flags for adoption info.")
@@ -252,6 +274,9 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=vsphere --vsphere-vcenter=vmware.
 	flags.StringVar(&opt.AdoptAdminUsername, "adopt-admin-username", "", "Username for cluster web console administrator. (optional)")
 	flags.StringVar(&opt.AdoptAdminPassword, "adopt-admin-password", "", "Password for cluster web console administrator. (optional)")
 
+	// AWS flags
+	flags.StringSliceVar(&opt.AWSUserTags, "aws-user-tags", nil, "Additional tags to add to resources. Must be in the form \"key=value\"")
+
 	// Azure flags
 	flags.StringVar(&opt.AzureBaseDomainResourceGroupName, "azure-base-domain-resource-group-name", "os4-common", "Resource group where the azure DNS zone for the base domain is found")
 
@@ -273,12 +298,43 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=vsphere --vsphere-vcenter=vmware.
 	flags.StringVar(&opt.VSphereNetwork, "vsphere-network", "", "Name of the network to be used by the cluster")
 	flags.StringVar(&opt.VSphereCACerts, "vsphere-ca-certs", "", "Path to vSphere CA certificate, multiple CA paths can be : delimited")
 
+	// oVirt flags
+	flags.StringVar(&opt.OvirtClusterID, "ovirt-cluster-id", "", "The oVirt cluster id (uuid) under which all VMs will run")
+	flags.StringVar(&opt.OvirtStorageDomainID, "ovirt-storage-domain-id", "", "oVirt storage domain id (uuid) under which all VM disk would be created")
+	flags.StringVar(&opt.OvirtNetworkName, "ovirt-network-name", "ovirtmgmt", "oVirt network name")
+	flags.StringVar(&opt.OvirtAPIVIP, "ovirt-api-vip", "", "IP which will be served by bootstrap and then pivoted masters, using keepalived")
+	flags.StringVar(&opt.OvirtIngressVIP, "ovirt-ingress-vip", "", "External IP which routes to the default ingress controller")
+	flags.StringVar(&opt.OvirtCACerts, "ovirt-ca-certs", "", "Path to oVirt CA certificate, multiple CA paths can be : delimited")
+
+	// Additional CA Trust Bundle
+	flags.StringVar(&opt.AdditionalTrustBundle, "additional-trust-bundle", "", "Path to a CA Trust Bundle which will be added to the nodes trusted certificate store.")
+
 	return cmd
 }
 
 // Complete finishes parsing arguments for the command
 func (o *Options) Complete(cmd *cobra.Command, args []string) error {
 	o.Name = args[0]
+
+	if o.Region == "" {
+		switch o.Cloud {
+		case cloudAWS:
+			o.Region = "us-east-1"
+		case cloudAzure:
+			o.Region = "centralus"
+		case cloudGCP:
+			o.Region = "us-east1"
+		}
+	}
+
+	if o.HibernateAfter != "" {
+		dur, err := time.ParseDuration(o.HibernateAfter)
+		if err != nil {
+			return errors.Wrapf(err, "unable to parse HibernateAfter duration")
+		}
+		o.HibernateAfterDur = &dur
+	}
+
 	return nil
 }
 
@@ -286,31 +342,31 @@ func (o *Options) Complete(cmd *cobra.Command, args []string) error {
 func (o *Options) Validate(cmd *cobra.Command) error {
 	if len(o.Output) > 0 && o.Output != "yaml" && o.Output != "json" {
 		cmd.Usage()
-		log.Info("Invalid value for output. Valid values are: yaml, json.")
+		o.log.Info("Invalid value for output. Valid values are: yaml, json.")
 		return fmt.Errorf("invalid output")
 	}
 	if !o.UseClusterImageSet && len(o.ClusterImageSet) > 0 {
 		cmd.Usage()
-		log.Info("If not using cluster image sets, do not specify the name of one")
+		o.log.Info("If not using cluster image sets, do not specify the name of one")
 		return fmt.Errorf("invalid option")
 	}
 	if len(o.ServingCert) > 0 && len(o.ServingCertKey) == 0 {
 		cmd.Usage()
-		log.Info("If specifying a serving certificate, specify a valid serving certificate key")
+		o.log.Info("If specifying a serving certificate, specify a valid serving certificate key")
 		return fmt.Errorf("invalid serving cert")
 	}
 	if !validClouds[o.Cloud] {
 		cmd.Usage()
-		log.Infof("Unsupported cloud: %s", o.Cloud)
+		o.log.Infof("Unsupported cloud: %s", o.Cloud)
 		return fmt.Errorf("unsupported cloud: %s", o.Cloud)
 	}
 	if o.Cloud == cloudOpenStack {
 		if o.OpenStackAPIFloatingIP == "" {
-			log.Info("Missing openstack-api-floating-ip parameter")
+			o.log.Info("Missing openstack-api-floating-ip parameter")
 			return fmt.Errorf("Missing openstack-api-floating-ip parameter")
 		}
 		if o.OpenStackCloud == "" {
-			log.Info("Missing openstack-cloud parameter")
+			o.log.Info("Missing openstack-cloud parameter")
 			return fmt.Errorf("Missing openstack-cloud parameter")
 		}
 	}
@@ -333,6 +389,21 @@ func (o *Options) Validate(cmd *cobra.Command) error {
 			return fmt.Errorf("cannot use adoption options without --adopt: --adopt-admin-kube-config, --adopt-infra-id, --adopt-cluster-id, --adopt-admin-username, --adopt-admin-password")
 		}
 	}
+
+	if o.Region != "" {
+		switch c := o.Cloud; c {
+		case cloudAWS, cloudAzure, cloudGCP:
+		default:
+			return fmt.Errorf("cannot specify region when cloud is %q", c)
+		}
+	}
+
+	for _, ls := range o.Labels {
+		tokens := strings.Split(ls, "=")
+		if len(tokens) != 2 {
+			return fmt.Errorf("unable to parse key=value label: %s", ls)
+		}
+	}
 	return nil
 }
 
@@ -356,21 +427,21 @@ func (o *Options) Run() error {
 		printObjects(objs, scheme.Scheme, printer)
 		return err
 	}
-	rh, err := o.getResourceHelper()
+	rh, err := utils.GetResourceHelper(o.log)
 	if err != nil {
 		return err
 	}
 	if len(o.Namespace) == 0 {
-		o.Namespace, err = o.defaultNamespace()
+		o.Namespace, err = utils.DefaultNamespace()
 		if err != nil {
-			log.Error("Cannot determine default namespace")
+			o.log.Error("Cannot determine default namespace")
 			return err
 		}
 	}
 	for _, obj := range objs {
 		accessor, err := meta.Accessor(obj)
 		if err != nil {
-			log.WithError(err).Errorf("Cannot create accessor for object of type %T", obj)
+			o.log.WithError(err).Errorf("Cannot create accessor for object of type %T", obj)
 			return err
 		}
 		accessor.SetNamespace(o.Namespace)
@@ -382,27 +453,10 @@ func (o *Options) Run() error {
 	return nil
 }
 
-func (o *Options) defaultNamespace() (string, error) {
-	rules := clientcmd.NewDefaultClientConfigLoadingRules()
-	kubeconfig := clientcmd.NewNonInteractiveDeferredLoadingClientConfig(rules, &clientcmd.ConfigOverrides{})
-	ns, _, err := kubeconfig.Namespace()
-	return ns, err
-}
-
-func (o *Options) getResourceHelper() (*resource.Helper, error) {
-	cfg, err := config.GetConfig()
-	if err != nil {
-		log.WithError(err).Error("Cannot get client config")
-		return nil, err
-	}
-	helper := resource.NewHelperFromRESTConfig(cfg, log.WithField("command", "create-cluster"))
-	return helper, nil
-}
-
 // GenerateObjects generates resources for a new cluster deployment
 func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 
-	pullSecret, err := o.getPullSecret()
+	pullSecret, err := utils.GetPullSecret(o.log, o.PullSecret, o.PullSecretFile)
 	if err != nil {
 		return nil, err
 	}
@@ -417,28 +471,42 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 		return nil, err
 	}
 
+	additionalTrustBundle, err := o.getAdditionalTrustBundle()
+	if err != nil {
+		return nil, err
+	}
+
 	// Load installer manifest files:
 	manifestFileData, err := o.getManifestFileBytes()
 	if err != nil {
 		return nil, err
 	}
 
+	labels := map[string]string{
+		hiveutilCreatedLabel: "true", // implied
+	}
+	for _, ls := range o.Labels {
+		tokens := strings.Split(ls, "=")
+		labels[tokens[0]] = tokens[1]
+	}
+
 	builder := &clusterresource.Builder{
-		Name:             o.Name,
-		Namespace:        o.Namespace,
-		WorkerNodesCount: o.WorkerNodesCount,
-		PullSecret:       pullSecret,
-		SSHPrivateKey:    sshPrivateKey,
-		SSHPublicKey:     sshPublicKey,
-		InstallOnce:      o.InstallOnce,
-		BaseDomain:       o.BaseDomain,
-		ManageDNS:        o.ManageDNS,
-		DeleteAfter:      o.DeleteAfter,
-		Labels: map[string]string{
-			hiveutilCreatedLabel: "true",
-		},
-		InstallerManifests: manifestFileData,
-		MachineNetwork:     o.MachineNetwork,
+		Name:                  o.Name,
+		Namespace:             o.Namespace,
+		WorkerNodesCount:      o.WorkerNodesCount,
+		PullSecret:            pullSecret,
+		SSHPrivateKey:         sshPrivateKey,
+		SSHPublicKey:          sshPublicKey,
+		InstallOnce:           o.InstallOnce,
+		BaseDomain:            o.BaseDomain,
+		ManageDNS:             o.ManageDNS,
+		DeleteAfter:           o.DeleteAfter,
+		HibernateAfter:        o.HibernateAfterDur,
+		Labels:                labels,
+		InstallerManifests:    manifestFileData,
+		MachineNetwork:        o.MachineNetwork,
+		SkipMachinePools:      o.SkipMachinePools,
+		AdditionalTrustBundle: additionalTrustBundle,
 	}
 	if o.Adopt {
 		kubeconfigBytes, err := ioutil.ReadFile(o.AdoptAdminKubeConfig)
@@ -460,21 +528,35 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 		if err != nil {
 			return nil, err
 		}
+		userTags := make(map[string]string, len(o.AWSUserTags))
+		for _, t := range o.AWSUserTags {
+			tagParts := strings.SplitN(t, "=", 2)
+			switch len(tagParts) {
+			case 0:
+			case 1:
+				userTags[tagParts[0]] = ""
+			case 2:
+				userTags[tagParts[0]] = tagParts[1]
+			}
+		}
 		awsProvider := &clusterresource.AWSCloudBuilder{
 			AccessKeyID:     accessKeyID,
 			SecretAccessKey: secretAccessKey,
+			UserTags:        userTags,
+			Region:          o.Region,
 		}
 		builder.CloudBuilder = awsProvider
 	case cloudAzure:
 		creds, err := azurecredutil.GetCreds(o.CredsFile)
 		if err != nil {
-			log.WithError(err).Error("Failed to read in Azure credentials")
+			o.log.WithError(err).Error("Failed to read in Azure credentials")
 			return nil, err
 		}
 
 		azureProvider := &clusterresource.AzureCloudBuilder{
 			ServicePrincipal:            creds,
 			BaseDomainResourceGroupName: o.AzureBaseDomainResourceGroupName,
+			Region:                      o.Region,
 		}
 		builder.CloudBuilder = azureProvider
 	case cloudGCP:
@@ -490,6 +572,7 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 		gcpProvider := &clusterresource.GCPCloudBuilder{
 			ProjectID:      projectID,
 			ServiceAccount: creds,
+			Region:         o.Region,
 		}
 		builder.CloudBuilder = gcpProvider
 	case cloudOpenStack:
@@ -576,6 +659,33 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 			CACert:           bytes.Join(caCerts, []byte("\n")),
 		}
 		builder.CloudBuilder = vsphereProvider
+	case cloudOVirt:
+		oVirtConfig, err := ovirtutils.GetCreds(o.CredsFile)
+		if err != nil {
+			return nil, err
+		}
+		if o.OvirtCACerts == "" {
+			return nil, errors.New("must provide --ovirt-ca-certs")
+		}
+		caCerts := [][]byte{}
+		for _, cert := range filepath.SplitList(o.OvirtCACerts) {
+			caCert, err := ioutil.ReadFile(cert)
+			if err != nil {
+				return nil, fmt.Errorf("error reading %s: %w", cert, err)
+			}
+			caCerts = append(caCerts, caCert)
+		}
+		oVirtProvider := &clusterresource.OvirtCloudBuilder{
+			OvirtConfig:     oVirtConfig,
+			ClusterID:       o.OvirtClusterID,
+			StorageDomainID: o.OvirtStorageDomainID,
+			NetworkName:     o.OvirtNetworkName,
+			APIVIP:          o.OvirtAPIVIP,
+			IngressVIP:      o.OvirtIngressVIP,
+			CACert:          bytes.Join(caCerts, []byte("\n")),
+		}
+		builder.CloudBuilder = oVirtProvider
+		builder.SkipMachinePools = true
 	}
 
 	if len(o.ServingCert) != 0 {
@@ -613,26 +723,6 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 	return result, nil
 }
 
-func (o *Options) getPullSecret() (string, error) {
-	pullSecret := os.Getenv("PULL_SECRET")
-	if len(pullSecret) > 0 {
-		return pullSecret, nil
-	}
-	if len(o.PullSecret) > 0 {
-		return o.PullSecret, nil
-	}
-	if len(o.PullSecretFile) > 0 {
-		data, err := ioutil.ReadFile(o.PullSecretFile)
-		if err != nil {
-			log.Error("Cannot read pull secret file")
-			return "", err
-		}
-		pullSecret = strings.TrimSpace(string(data))
-		return pullSecret, nil
-	}
-	return "", nil
-}
-
 func (o *Options) getSSHPublicKey() (string, error) {
 	sshPublicKey := os.Getenv("PUBLIC_SSH_KEY")
 	if len(sshPublicKey) > 0 {
@@ -644,14 +734,14 @@ func (o *Options) getSSHPublicKey() (string, error) {
 	if len(o.SSHPublicKeyFile) > 0 {
 		data, err := ioutil.ReadFile(o.SSHPublicKeyFile)
 		if err != nil {
-			log.Error("Cannot read SSH public key file")
+			o.log.Error("Cannot read SSH public key file")
 			return "", err
 		}
 		sshPublicKey = strings.TrimSpace(string(data))
 		return sshPublicKey, nil
 	}
 
-	log.Error("Cannot determine SSH key to use")
+	o.log.Error("Cannot determine SSH key to use")
 	return "", nil
 }
 
@@ -659,13 +749,31 @@ func (o *Options) getSSHPrivateKey() (string, error) {
 	if len(o.SSHPrivateKeyFile) > 0 {
 		data, err := ioutil.ReadFile(o.SSHPrivateKeyFile)
 		if err != nil {
-			log.Error("Cannot read SSH private key file")
+			o.log.Error("Cannot read SSH private key file")
 			return "", err
 		}
 		sshPrivateKey := strings.TrimSpace(string(data))
 		return sshPrivateKey, nil
 	}
-	log.Debug("No private SSH key file provided")
+	o.log.Debug("No private SSH key file provided")
+	return "", nil
+}
+
+func (o *Options) getAdditionalTrustBundle() (string, error) {
+	if len(o.AdditionalTrustBundle) > 0 {
+		data, err := ioutil.ReadFile(o.AdditionalTrustBundle)
+		if err != nil {
+			o.log.Error("Cannot read AdditionalTrustBundle file")
+			return "", err
+		}
+		if err := validate.CABundle(string(data)); err != nil {
+			o.log.Error("AdditionalTrustBundle is not valid")
+			return "", err
+		}
+		additionalTrustBundle := string(data)
+		return additionalTrustBundle, nil
+	}
+	o.log.Debug("No AdditionalTrustBundle provided")
 	return "", nil
 }
 
@@ -707,7 +815,7 @@ func (o *Options) configureImages(generator *clusterresource.Builder) (*hivev1.C
 			return nil, fmt.Errorf("specify either a release image or a release image source")
 		}
 		var err error
-		o.ReleaseImage, err = determineReleaseImageFromSource(o.ReleaseImageSource)
+		o.ReleaseImage, err = utils.DetermineReleaseImageFromSource(o.ReleaseImageSource)
 		if err != nil {
 			return nil, fmt.Errorf("cannot determine release image: %v", err)
 		}
@@ -830,25 +938,3 @@ func printObjects(objects []runtime.Object, scheme *runtime.Scheme, printer prin
 		typeSetterPrinter.PrintObj(list, os.Stdout)
 	}
 }
-
-type releasePayload struct {
-	PullSpec string `json:"pullSpec"`
-}
-
-func determineReleaseImageFromSource(sourceURL string) (string, error) {
-	resp, err := http.Get(sourceURL)
-	if err != nil {
-		return "", err
-	}
-	defer resp.Body.Close()
-	data, err := ioutil.ReadAll(resp.Body)
-	if err != nil {
-		return "", err
-	}
-	payload := &releasePayload{}
-	err = json.Unmarshal(data, payload)
-	if err != nil {
-		return "", err
-	}
-	return payload.PullSpec, nil
-}
