@@ -19,6 +19,7 @@ import (
 	hivev1azure "github.com/openshift/hive/pkg/apis/hive/v1/azure"
 	hivev1gcp "github.com/openshift/hive/pkg/apis/hive/v1/gcp"
 	hivev1openstack "github.com/openshift/hive/pkg/apis/hive/v1/openstack"
+	hivev1ovirt "github.com/openshift/hive/pkg/apis/hive/v1/ovirt"
 	hivev1vsphere "github.com/openshift/hive/pkg/apis/hive/v1/vsphere"
 	"github.com/openshift/hive/pkg/constants"
 )
@@ -79,6 +80,20 @@ func validAWSClusterDeployment() *hivev1.ClusterDeployment {
 	return cd
 }
 
+func validAWSClusterDeploymentFromPool(poolNS, poolName, claimName string) *hivev1.ClusterDeployment {
+	cd := clusterDeploymentTemplate()
+	cd.Spec.Platform.AWS = &hivev1aws.Platform{
+		CredentialsSecretRef: corev1.LocalObjectReference{Name: "fake-creds-secret"},
+		Region:               "test-region",
+	}
+	cd.Spec.ClusterPoolRef = &hivev1.ClusterPoolReference{
+		Namespace: poolNS,
+		PoolName:  poolName,
+		ClaimName: claimName,
+	}
+	return cd
+}
+
 func validAzureClusterDeployment() *hivev1.ClusterDeployment {
 	cd := clusterDeploymentTemplate()
 	cd.Spec.Platform.Azure = &hivev1azure.Platform{
@@ -113,6 +128,17 @@ func validVSphereClusterDeployment() *hivev1.ClusterDeployment {
 	return cd
 }
 
+func validOvirtClusterDeployment() *hivev1.ClusterDeployment {
+	cd := clusterDeploymentTemplate()
+	cd.Spec.Platform.Ovirt = &hivev1ovirt.Platform{
+		ClusterID:             "fake-cluster-uuid",
+		CredentialsSecretRef:  corev1.LocalObjectReference{Name: "fake-creds-secret"},
+		CertificatesSecretRef: corev1.LocalObjectReference{Name: "fake-cert-secret"},
+		StorageDomainID:       "fake-storage-domain-uuid",
+	}
+	return cd
+}
+
 // Meant to be used to compare new and old as the same values.
 func validClusterDeploymentSameValues() *hivev1.ClusterDeployment {
 	return validAWSClusterDeployment()
@@ -220,6 +246,60 @@ func TestClusterDeploymentValidate(t *testing.T) {
 			operation:       admissionv1beta1.Update,
 			expectedAllowed: false,
 		},
+		{
+			name:            "Test create with ClusterPoolReference",
+			newObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", ""),
+			operation:       admissionv1beta1.Create,
+			expectedAllowed: true,
+		},
+		{
+			name:            "Test create with Claimed ClusterPoolReference",
+			newObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", "test-claim"),
+			operation:       admissionv1beta1.Create,
+			expectedAllowed: false,
+		},
+		{
+			name:            "Test update with removed ClusterPoolReference",
+			oldObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", ""),
+			newObject:       validAWSClusterDeployment(),
+			operation:       admissionv1beta1.Update,
+			expectedAllowed: false,
+		},
+		{
+			name:            "Test update with added ClusterPoolReference",
+			oldObject:       validAWSClusterDeployment(),
+			newObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", ""),
+			operation:       admissionv1beta1.Update,
+			expectedAllowed: false,
+		},
+		{
+			name:            "Test update with modified ClusterPoolReference",
+			oldObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", ""),
+			newObject:       validAWSClusterDeploymentFromPool("new-pool-ns", "new-mypool", ""),
+			operation:       admissionv1beta1.Update,
+			expectedAllowed: false,
+		},
+		{
+			name:            "Test update with claimed ClusterPoolReference",
+			oldObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", ""),
+			newObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", "test-claim"),
+			operation:       admissionv1beta1.Update,
+			expectedAllowed: true,
+		},
+		{
+			name:            "Test update with unclaimed ClusterPoolReference",
+			oldObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", "test-claim"),
+			newObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", ""),
+			operation:       admissionv1beta1.Update,
+			expectedAllowed: false,
+		},
+		{
+			name:            "Test update with changed claim",
+			oldObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", "test-claim"),
+			newObject:       validAWSClusterDeploymentFromPool("pool-ns", "mypool", "other-claim"),
+			operation:       admissionv1beta1.Update,
+			expectedAllowed: false,
+		},
 		{
 			name:      "Test setting installed flag without cluster metadata",
 			oldObject: validAWSClusterDeployment(),
@@ -376,11 +456,23 @@ func TestClusterDeploymentValidate(t *testing.T) {
 			expectedAllowed: true,
 		},
 		{
-			name:            "Test invalid managed domain",
+			name:            "Test base domain is not child of a managed domain",
+			newObject:       clusterDeploymentWithManagedDomain("bar.bad-domain.com"),
+			operation:       admissionv1beta1.Create,
+			expectedAllowed: false,
+		},
+		{
+			name:            "Test base domain is not direct child of a managed domain",
 			newObject:       clusterDeploymentWithManagedDomain("baz.foo.bbb.com"),
 			operation:       admissionv1beta1.Create,
 			expectedAllowed: false,
 		},
+		{
+			name:            "Test base domain is not same as one of the managed domains",
+			newObject:       clusterDeploymentWithManagedDomain("foo.aaa.com"),
+			operation:       admissionv1beta1.Create,
+			expectedAllowed: false,
+		},
 		{
 			name: "Test managed DNS is valid on GCP",
 			newObject: func() *hivev1.ClusterDeployment {
@@ -718,6 +810,12 @@ func TestClusterDeploymentValidate(t *testing.T) {
 			operation:       admissionv1beta1.Create,
 			expectedAllowed: true,
 		},
+		{
+			name:            "oVirt create valid",
+			newObject:       validOvirtClusterDeployment(),
+			operation:       admissionv1beta1.Create,
+			expectedAllowed: true,
+		},
 	}
 
 	for _, tc := range cases {
