@@ -131,6 +131,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -152,6 +153,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			machinePool:          testMachinePool(),
 			actuatorDoNotProceed: true,
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -172,6 +174,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 0, 0),
@@ -192,6 +195,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 			},
@@ -231,6 +235,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -252,6 +257,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 3, 0),
 				testMachineSet("foo-12345-other-us-east-1b", "other", true, 3, 0),
 			},
@@ -268,6 +274,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-other-us-east-1a", "other", true, 1, 0),
 				testMachineSet("foo-12345-other-us-east-1b", "other", true, 1, 0),
 				testMachineSet("foo-12345-other-us-east-1c", "other", true, 1, 0),
@@ -296,6 +303,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				return mp
 			}(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-other-us-east-1a", "other", true, 1, 0),
 				testMachineSet("foo-12345-other-us-east-1b", "other", true, 1, 0),
 				testMachineSet("foo-12345-other-us-east-1c", "other", true, 1, 0),
@@ -314,6 +322,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			name:        "No cluster deployment",
 			machinePool: testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -335,6 +344,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			}(),
 			machinePool: testMachinePool(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -351,6 +361,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -383,6 +394,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -414,6 +426,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -450,6 +463,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -486,6 +500,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -515,6 +530,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -547,6 +563,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			clusterDeployment: testClusterDeployment(),
 			machinePool:       testAutoscalingMachinePool(3, 5),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -585,6 +602,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				return mp
 			}(),
 			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
 				testMachineSet("foo-12345-worker-us-east-1a", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1b", "worker", true, 1, 0),
 				testMachineSet("foo-12345-worker-us-east-1c", "worker", true, 1, 0),
@@ -637,7 +655,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				scheme:                        scheme.Scheme,
 				logger:                        logger,
 				remoteClusterAPIClientBuilder: func(*hivev1.ClusterDeployment) remoteclient.Builder { return mockRemoteClientBuilder },
-				actuatorBuilder: func(cd *hivev1.ClusterDeployment, pool *hivev1.MachinePool, remoteMachineSets []machineapi.MachineSet, cdLog log.FieldLogger) (Actuator, error) {
+				actuatorBuilder: func(cd *hivev1.ClusterDeployment, pool *hivev1.MachinePool, masterMachine *machineapi.Machine, remoteMachineSets []machineapi.MachineSet, cdLog log.FieldLogger) (Actuator, error) {
 					return mockActuator, nil
 				},
 				expectations: controllerExpectations,
@@ -682,7 +700,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 								t.Errorf("machineset %v has unexpected labels:\nexpected: %v\nactual: %v", eMS.Name, eMS.Labels, rMS.Labels)
 							}
 							if !reflect.DeepEqual(eMS.ObjectMeta.Annotations, rMS.ObjectMeta.Annotations) {
-								t.Errorf("machineset %v has unexpected annotations:\nexpected: %v\nactual: %v", eMS.Name, eMS.Labels, rMS.Labels)
+								t.Errorf("machineset %v has unexpected annotations:\nexpected: %v\nactual: %v", eMS.Name, eMS.Annotations, rMS.Annotations)
 							}
 							if !reflect.DeepEqual(eMS.Spec.Template.Spec.Labels, rMS.Spec.Template.Spec.Labels) {
 								t.Errorf("machineset %v machinespec has unexpected labels:\nexpected: %v\nactual: %v", eMS.Name, eMS.Spec.Template.Spec.Labels, rMS.Spec.Template.Spec.Labels)
@@ -780,9 +798,8 @@ func testAutoscalingMachinePool(min, max int) *hivev1.MachinePool {
 	return p
 }
 
-func testMachineSet(name string, machineType string, unstompedAnnotation bool, replicas int, generation int) *machineapi.MachineSet {
-	msReplicas := int32(replicas)
-	awsProviderSpec := &awsprovider.AWSMachineProviderConfig{
+func testAWSProviderSpec() *awsprovider.AWSMachineProviderConfig {
+	return &awsprovider.AWSMachineProviderConfig{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "AWSMachineProviderConfig",
 			APIVersion: awsprovider.SchemeGroupVersion.String(),
@@ -791,11 +808,46 @@ func testMachineSet(name string, machineType string, unstompedAnnotation bool, r
 			ID: aws.String(testAMI),
 		},
 	}
-	rawAWSProviderSpec, err := encodeAWSMachineProviderSpec(awsProviderSpec, scheme.Scheme)
+}
+
+func testMachineSpec(machineType string) machineapi.MachineSpec {
+	rawAWSProviderSpec, err := encodeAWSMachineProviderSpec(testAWSProviderSpec(), scheme.Scheme)
 	if err != nil {
 		log.WithError(err).Fatal("error encoding AWS machine provider spec")
 	}
+	return machineapi.MachineSpec{
+		ObjectMeta: metav1.ObjectMeta{
+			Labels: map[string]string{
+				"machine.openshift.io/cluster-api-cluster":      testInfraID,
+				"machine.openshift.io/cluster-api-machine-role": machineType,
+				"machine.openshift.io/cluster-api-machine-type": machineType,
+			},
+		},
+		ProviderSpec: machineapi.ProviderSpec{
+			Value: rawAWSProviderSpec,
+		},
 
+		Taints: []corev1.Taint{
+			{
+				Key:    "foo",
+				Value:  "bar",
+				Effect: corev1.TaintEffectNoSchedule,
+			},
+		},
+	}
+}
+
+func testMachine(name string, machineType string) *machineapi.Machine {
+	return &machineapi.Machine{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+		},
+		Spec: testMachineSpec(machineType),
+	}
+}
+
+func testMachineSet(name string, machineType string, unstompedAnnotation bool, replicas int, generation int) *machineapi.MachineSet {
+	msReplicas := int32(replicas)
 	ms := machineapi.MachineSet{
 		TypeMeta: metav1.TypeMeta{
 			APIVersion: machineapi.SchemeGroupVersion.String(),
@@ -807,32 +859,14 @@ func testMachineSet(name string, machineType string, unstompedAnnotation bool, r
 			Labels: map[string]string{
 				machinePoolNameLabel:                       machineType,
 				"machine.openshift.io/cluster-api-cluster": testInfraID,
+				constants.HiveManagedLabel:                 "true",
 			},
 			Generation: int64(generation),
 		},
 		Spec: machineapi.MachineSetSpec{
 			Replicas: &msReplicas,
 			Template: machineapi.MachineTemplateSpec{
-				Spec: machineapi.MachineSpec{
-					ObjectMeta: metav1.ObjectMeta{
-						Labels: map[string]string{
-							"machine.openshift.io/cluster-api-cluster":      testInfraID,
-							"machine.openshift.io/cluster-api-machine-role": machineType,
-							"machine.openshift.io/cluster-api-machine-type": machineType,
-						},
-					},
-					ProviderSpec: machineapi.ProviderSpec{
-						Value: rawAWSProviderSpec,
-					},
-
-					Taints: []corev1.Taint{
-						{
-							Key:    "foo",
-							Value:  "bar",
-							Effect: corev1.TaintEffectNoSchedule,
-						},
-					},
-				},
+				Spec: testMachineSpec(machineType),
 			},
 		},
 	}
@@ -892,6 +926,9 @@ func testClusterDeployment() *hivev1.ClusterDeployment {
 			Namespace:  testNamespace,
 			Finalizers: []string{hivev1.FinalizerDeprovision},
 			UID:        types.UID("1234"),
+			Labels: map[string]string{
+				constants.VersionMajorMinorPatchLabel: "4.4.0",
+			},
 		},
 		Spec: hivev1.ClusterDeploymentSpec{
 			ClusterName: testName,
@@ -926,3 +963,11 @@ func printAWSMachineProviderConfig(cfg *awsprovider.AWSMachineProviderConfig) st
 	}
 	return string(b)
 }
+
+func withClusterVersion(cd *hivev1.ClusterDeployment, version string) *hivev1.ClusterDeployment {
+	if cd.Labels == nil {
+		cd.Labels = make(map[string]string, 1)
+	}
+	cd.Labels[constants.VersionMajorMinorPatchLabel] = version
+	return cd
+}
